@@ -1,6 +1,13 @@
 import datetime
 import json
 import os
+import zipfile
+try:
+    from StringIO import StringIO ## for Python 2
+except ImportError:
+    from io import StringIO ## for Python 3
+
+from capstone import Cs, CS_ARCH_X86, CS_MODE_32, CS_MODE_64
 
 from smda.DisassemblyStatistics import DisassemblyStatistics
 from .SmdaFunction import SmdaFunction
@@ -14,6 +21,7 @@ class SmdaReport(object):
     binary_size = None
     binweight = None
     bitness = None
+    buffer = None
     code_areas = None
     component = None
     confidence_threshold = None
@@ -34,13 +42,17 @@ class SmdaReport(object):
     version = None
     xcfg = None
 
-    def __init__(self, disassembly=None, config=None):
+    # in case we need to re-disassemble with more detail, we hold an initialized capstone instance as singleton
+    capstone = None
+
+    def __init__(self, disassembly=None, config=None, buffer=None):
         if disassembly is not None:
             self.architecture = disassembly.binary_info.architecture
             self.base_addr = disassembly.binary_info.base_addr
             self.binary_size = disassembly.binary_info.binary_size
             self.binweight = 0
             self.bitness = disassembly.binary_info.bitness
+            self.buffer = buffer
             self.code_areas = disassembly.binary_info.code_areas
             self.component = disassembly.binary_info.component
             self.confidence_threshold = disassembly.getConfidenceThreshold()
@@ -67,7 +79,7 @@ class SmdaReport(object):
         for function_offset in disassembly.functions:
             if self.confidence_threshold and function_offset in disassembly.candidates and disassembly.candidates[function_offset].getConfidence() < self.confidence_threshold:
                 continue
-            smda_function = SmdaFunction(disassembly, function_offset, config=config)
+            smda_function = SmdaFunction(disassembly, function_offset, config=config, smda_report=self)
             function_results[function_offset] = smda_function
             self.binweight += smda_function.binweight
         return function_results
@@ -90,6 +102,8 @@ class SmdaReport(object):
             sum_instructions += function.num_instructions
         return sum_instructions
 
+    def getBuffer(self):
+        return self.buffer
 
     def getFunction(self, function_addr):
         return self.xcfg[function_addr] if function_addr in self.xcfg else None
@@ -98,6 +112,29 @@ class SmdaReport(object):
         for _, smda_function in sorted(self.xcfg.items()):
             yield smda_function
 
+    def getCapstone(self):
+        if self.capstone is None:
+            self.capstone = Cs(CS_ARCH_X86, CS_MODE_64) if self.bitness == 64 else Cs(CS_ARCH_X86, CS_MODE_32)
+            self.capstone.detail = True
+        return self.capstone
+
+    def isAddrWithinMemoryImage(self, offset):
+        return self.base_addr <= offset < self.base_addr + self.binary_size
+
+    def _packBuffer(self, buffer):
+        # TODO
+        # create zip
+        # XOR with some key
+        # base64
+        return b""
+
+    def _unpackBuffer(self, buffer):
+        # TODO
+        # de-base64
+        # XOR with some key
+        # read from zip
+        return b""
+
     @classmethod
     def fromFile(cls, file_path):
         smda_json = {}
@@ -142,7 +179,7 @@ class SmdaReport(object):
         binary_info.architecture = smda_report.architecture
         binary_info.base_addr = smda_report.base_addr
         binary_info.binary_size = smda_report.binary_size
-        smda_report.xcfg = {int(function_addr): SmdaFunction.fromDict(function_dict, binary_info=binary_info, version=smda_report.smda_version) for function_addr, function_dict in report_dict["xcfg"].items()}
+        smda_report.xcfg = {int(function_addr): SmdaFunction.fromDict(function_dict, binary_info=binary_info, version=smda_report.smda_version, smda_report=smda_report) for function_addr, function_dict in report_dict["xcfg"].items()}
         return smda_report
 
     def toDict(self):
