@@ -119,17 +119,17 @@ class FunctionCandidateManager(object):
                 yield candidate
 
     def _logCandidateStats(self):
-        logging.debug("Candidate Statistics:")
+        LOGGER.debug("Candidate Statistics:")
         try:
             maxc = max([c.getScore() for c in self.candidates.values()])
             minc = min([c.getScore() for c in self.candidates.values()])
             candidates_2 = len([c.getScore() for c in self.candidates.values() if c.getScore() == 2])
             candidates_1 = len([c.getScore() for c in self.candidates.values() if c.getScore() == 1])
             candidates_0 = len([c.getScore() for c in self.candidates.values() if c.getScore() == 0])
-            logging.debug("  Max: %f, Min: %f", maxc, minc)
-            logging.debug("  2: %d, 1: %d, 0: %d", candidates_2, candidates_1, candidates_0)
+            LOGGER.debug("  Max: %f, Min: %f", maxc, minc)
+            LOGGER.debug("  2: %d, 1: %d, 0: %d", candidates_2, candidates_1, candidates_0)
         except:
-            logging.debug("  No candidates found.")
+            LOGGER.debug("  No candidates found.")
 
     def getFunctionStartCandidates(self):
         return self._candidate_offsets
@@ -224,13 +224,15 @@ class FunctionCandidateManager(object):
                 self.gap_pointer += 1
                 continue
             # try to find instructions that directly encode as NOP and skip them
-            ins_buf = [i for i in self.capstone.disasm(self.disassembly.getRawBytes(gap_offset, 15), gap_offset)]
-            if ins_buf and ins_buf[0].mnemonic == "nop":
-                nop_instruction = ins_buf[0].mnemonic + " " + ins_buf[0].op_str
-                nop_length = len(ins_buf[0].bytes)
-                LOGGER.debug("nextGapCandidate() found nop instruction (%s) - gap_ptr += %d: 0x%08x", nop_instruction, nop_length, self.gap_pointer)
-                self.gap_pointer += nop_length
-                continue
+            ins_buf = [i for i in self.capstone.disasm_lite(self.disassembly.getRawBytes(gap_offset, 15), gap_offset)]
+            if ins_buf:
+                i_address, i_size, i_mnemonic, i_op_str = ins_buf[0]
+                if  i_mnemonic == "nop":
+                    nop_instruction = i_mnemonic + " " + i_op_str
+                    nop_length = i_size
+                    LOGGER.debug("nextGapCandidate() found nop instruction (%s) - gap_ptr += %d: 0x%08x", nop_instruction, nop_length, self.gap_pointer)
+                    self.gap_pointer += nop_length
+                    continue
             # try to find effective NOPs and skip them.
             found_multi_byte_nop = False
             for gap_length in range(max(GAP_SEQUENCES.keys()), 1, -1):
